@@ -234,8 +234,12 @@ circos.initialize = function(factors, x = NULL, xlim = NULL, sector.width = NULL
             factors = factor(factors)
         }
     }
-    levels(factors) = intersect(levels(factors), as.character(factors))
+    factors = factor(as.character(factors), intersect(levels(factors), as.character(factors)))
     le = levels(factors)
+
+    if(!is.null(x)) {
+    	x = as.numeric(x)
+    }
     
     # initialize .SECTOR.DATA
     # you can think it as the global x axis configuration
@@ -246,6 +250,8 @@ circos.initialize = function(factors, x = NULL, xlim = NULL, sector.width = NULL
         if(length(xlim) != 2) {
             stop("Since `xlim` is vector, it should have length of 2.")
         }    
+
+        xlim = as.numeric(xlim)
         
         min.value = rep(xlim[1], length(le))
         max.value = rep(xlim[2], length(le))
@@ -259,6 +265,11 @@ circos.initialize = function(factors, x = NULL, xlim = NULL, sector.width = NULL
         		xlim = xlim[le, ,drop = FALSE]
         	}
         }
+        if(is.data.frame(xlim)) xlim = as.matrix(xlim)
+        xlim2 = as.numeric(xlim)
+    	dim(xlim2) = dim(xlim)
+    	dimnames(xlim2) = dimnames(xlim)
+    	xlim = xlim2
         
         min.value = apply(xlim, 1, function(x) x[1])
         max.value = apply(xlim, 1, function(x) x[2])
@@ -782,7 +793,7 @@ get.cell.meta.data = function(name, sector.index = get.current.sector.index(),
 }
 
 # == title (variable:CELL_META)
-# Easy to way to get meta data in the current cell
+# Easy way to get meta data in the current cell
 #
 # == details
 # The variable `CELL_META` can only be used to get meta data of the "current" cell.
@@ -801,6 +812,15 @@ get.cell.meta.data = function(name, sector.index = get.current.sector.index(),
 # dev.off()
 CELL_META = "don't use me directly"
 class(CELL_META) = "CELL_META"
+
+# == title 
+# Names of all meta data in the current cell
+#
+# == param
+# -x use `CELL_META`.
+#
+# == example
+# names(CELL_META)
 names.CELL_META = function(x) {
 	c("xlim", "ylim", "xrange", "yrange", "xcenter", "ycenter", "cell.xlim", "cell.ylim",
      "sector.numeric.index", "sector.index", "track.index", "xplot", "yplot", "track.margin", "cell.padding",
